/*
 * (C) Copyright 2013, Texas Instruments Incorporated.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <xdc/std.h>
#include <xdc/runtime/System.h>

#include <string.h>

#include <ti/sysbios/family/c66/Cache.h>
#include <ti/sdo/fc/edmamgr/edmamgr.h>

#include "edmamgr_test.h"

/*******************************************************************************
 * FUNCTION PURPOSE: Test for the maximum number of channels that can be
                     allocated.
 *******************************************************************************
   DESCRIPTION: This function allocated as many edmaMgr channels which is
                possible.
 ******************************************************************************/
int32_t EdmaMgr_test_maxChannels(int32_t core_id)
{
  EdmaMgr_Handle edmaMgrHandles[128];
  int32_t i = 0, ret_val, errors = 0;

  System_printf("EdmaMgr maxChannels Test: See how many channels may be allocated.\n");

  /* Allocate channels until there are no more free */
  for ( i = 0; i < 128; i++)
  {
    edmaMgrHandles[i] = EdmaMgr_alloc(1);

    if (edmaMgrHandles[i] != (EdmaMgr_Handle)NULL)
    {
      System_printf("\tAllocated channel %d\n", i);
    }
    else
    {
      break;
    }
  }

  System_printf("\n\tAllocated %d channels.\n\n", i);
//  System_printf("Paused...\n\n"); getchar();

  /* Free allocated channels */
  while ( i > 0 )
  {
    i--;
    System_printf("\tFree channel %d = ", i);

    ret_val = EdmaMgr_free(edmaMgrHandles[i]);

    if ( ret_val != 0)
    {
      System_printf("FAILED\n");
      errors++;
    }
    else
    {
      System_printf("SUCCESS\n");
    }
  }

  System_printf("\nEdmaMgr maxChannels Test: %d errors\n\n", errors);

  return (errors);
}
